@@ -1,5 +1,5 @@
 #
-# Copyright 2014-2017 Ghent University
+# Copyright 2014-2020 Ghent University
 #
 # This file is part of vsc-install,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -33,12 +33,22 @@ TestCase: use instead of unittest TestCase
 @author: Kenneth Hoste (Ghent University)
 """
 import pprint
+import os
 import re
+import shutil
 import sys
+import tempfile
 
-from cStringIO import StringIO
+try:
+    from cStringIO import StringIO  # Python 2
+except ImportError:
+    from io import StringIO  # Python 3
+
+from copy import deepcopy
 from unittest import TestCase as OrigTestCase
 from vsc.install.headers import nicediff
+from vsc.install.methodinspector import MethodInspector
+from mock import patch
 
 
 class TestCase(OrigTestCase):
@@ -51,8 +61,17 @@ class TestCase(OrigTestCase):
     ASSERT_MAX_DIFF = 100
     DIFF_OFFSET = 5 # lines of text around changes
 
+    def is_string(self, x):
+        """test if the variable x is a string)"""
+        try:
+            return isinstance(x, basestring)
+        except NameError:
+            return isinstance(x, str)
+
+    # pylint: disable=arguments-differ
     def assertEqual(self, a, b, msg=None):
         """Make assertEqual always print useful messages"""
+
         try:
             super(TestCase, self).assertEqual(a, b)
         except AssertionError as e:
@@ -61,11 +80,11 @@ class TestCase(OrigTestCase):
             else:
                 msg = "%s: %s" % (msg, e)
 
-            if isinstance(a, basestring):
+            if self.is_string(a):
                 txta = a
             else:
                 txta = pprint.pformat(a)
-            if isinstance(b, basestring):
+            if self.is_string(b):
                 txtb = b
             else:
                 txtb = pprint.pformat(b)
@@ -88,6 +107,12 @@ class TestCase(OrigTestCase):
         self.orig_sys_stdout = sys.stdout
         self.orig_sys_stderr = sys.stderr
 
+        self.orig_sys_argv = sys.argv
+        sys.argv = deepcopy(self.orig_sys_argv)
+
+        self.orig_workdir = os.getcwd()
+        self.tmpdir = tempfile.mkdtemp()
+
     def convert_exception_to_str(self, err):
         """Convert an Exception instance to a string."""
         msg = err
@@ -117,11 +142,11 @@ class TestCase(OrigTestCase):
         try:
             call(*args, **kwargs)
             str_kwargs = ['='.join([k, str(v)]) for (k, v) in kwargs.items()]
-            str_args = ', '.join(map(str, args) + str_kwargs)
+            str_args = ', '.join(list(map(str, args)) + str_kwargs)
             self.assertTrue(False, "Expected errors with %s(%s) call should occur" % (call.__name__, str_args))
-        except error, err:
+        except error as err:
             msg = self.convert_exception_to_str(err)
-            if isinstance(regex, basestring):
+            if self.is_string(regex):
                 regex = re.compile(regex)
             self.assertTrue(regex.search(msg), "Pattern '%s' is found in '%s'" % (regex.pattern, msg))
 
@@ -160,6 +185,8 @@ class TestCase(OrigTestCase):
                 funcname = logmethod_func.func_name
             elif hasattr(logmethod_func, 'im_func'):
                 funcname = logmethod_func.im_func.__name__
+            elif hasattr(logmethod_func, '__name__'):
+                funcname = logmethod_func.__name__
             else:
                 raise Exception("Unknown logmethod %s" % (dir(logmethod_func)))
             logcache = self.LOGCACHE.setdefault(funcname, [])
@@ -185,11 +212,32 @@ class TestCase(OrigTestCase):
         """
         return len(self.LOGCACHE.get(funcname, []))
 
+    def gen_inspector(self, *args, **kwargs):
+        """
+        Convenience method to generate MethodInspector instance.
+        All args are passed to the MethodInspector
+        """
+        return MethodInspector(*args, **kwargs)
+
+    def create_patch(self, *args, **kwargs):
+        """
+        Create patch and return mocked whatever. Patch is added to tearDown
+        All args are passed to patch
+        """
+        patcher = patch(*args, **kwargs)
+        thing = patcher.start()
+        self.addCleanup(patcher.stop)
+        return thing
+
     def tearDown(self):
         """Cleanup after running a test."""
         self.mock_stdout(False)
         self.mock_stderr(False)
         self.reset_logcache()
+        sys.argv = self.orig_sys_argv
+        os.chdir(self.orig_workdir)
+        shutil.rmtree(self.tmpdir)
+
         super(TestCase, self).tearDown()
 
 
